let boardGrid = []; let boardSize = -1;
let newX = -1; let newY = -1;
let gameStarted = false; let gameEnded = false;
let gameWin = false; let gameLose = false;
// Create two dimensional array
for (i = 0; i < 6; i++) {
  boardGrid[i] = [];
  for (j = 0; j < 6; j++) {
    boardGrid[i][j] = -1;
  }
}
let tileValues = new Array(0, 2, 4, 8, 16, 32, 64, 128, 256, 1024, 2048);
let tileColors = new Array("White", "DodgerBlue", "DeepSkyBlue", "Cyan", "Aquamarine", "Lime", "GreenYellow", "Yellow", "Orange", "DarkOrange", "Red", "DarkRed");
let tileFontSizes = new Array(30, 25, 20, 15);

// Select the board layout (3x3 .. 6x6)

function boardSelect (bsize) {
  // This function is called by pushing one of the "board selection" buttons on the webpage
  boardSize = bsize;                                                           // actual board size
  boardInit();                                                                 // init the board for this game
  boardDisplay();                                                              // display the initial (empty) board
  gameStarted = false; gameEnded = false;
  // Make board with actual size (table row) visible
  if (document.getElementById) {
    let board = 'b' + bsize;
    document.getElementById('b3').style.visibility = "collapse";
    document.getElementById('b4').style.visibility = "collapse";
    document.getElementById('b5').style.visibility = "collapse";
    document.getElementById('b6').style.visibility = "collapse";
    document.getElementById(board).style.visibility = "visible";
  }
}

// Init the actual board (set all array elements to zero)

function boardInit() {
  for (i = 0; i < 6; i++) {
    for (j = 0; j < 6; j++) {
      if (i < boardSize && j < boardSize) {
        boardGrid[i][j] = 0;
      }
      else {
        boardGrid[i][j] = -1;
      }
    }
  }
}

// Start the game

function gameStart() {
  // This function is called by pushing the "Start" button on the webpage
  if (!gameStarted && !gameEnded)   {
    // Only do if the game has not yet been started (and has not ended)
    gameStarted = true;
    tilesNew();                                                                // generate the first "2" tile
    boardDisplay();                                                            // display the board (with this tile)
  }
}

// Generate a new "2" tile at a random board position

function tilesNew() {
  let found = false;
  do {
    newX = Math.floor(Math.random() * boardSize);
    newY = Math.floor(Math.random() * boardSize);
    if (boardGrid[newX][newY] == 0) {
      // Check if generated position is actually empty
      found = true;
      boardGrid[newX][newY] = 2;
    }
  } while (found == false);
}

// Display the board (with actual tile values)

function boardDisplay() {
  for (i = 0; i < boardSize; i++) {
    for (j = 0; j < boardSize; j++) {
      let tileId = 'b' + boardSize + '_' + i + j;                              // id attribute for actual table element (<td>)
      if (document.getElementById) {
        if (boardGrid[i][j] == 0) {
          // Empty place
          document.getElementById(tileId).firstChild.data = "";
        }
        else {
          // Tile with given value
          document.getElementById(tileId).firstChild.data = boardGrid[i][j];
        }
        // Determine tile color (depending on its value)
        let tileColor = "White";
        for (k = 0; k < 12; k++) {
          // Find index of actual value (that is the same as the index of the colors array)
          if (boardGrid[i][j] == tileValues[k]) {
            tileColor = tileColors[k];
            if (boardGrid[i][j] == 2 && i == newX && j == newY) {
              tileColor = "Blue";                                              // extra color for the new tile
            }
          }
        }
        // Construct the style attribute of the <td> corresponding to this tile
        // Take the color from the colors array (using the index determined before)
        // Take the font size from the font sizes array (with index derived from board size)
        let tileStyle = "background-color:" + tileColor + "; font-size:" + tileFontSizes[boardSize - 3];
        // Apply the style to the <td> element
        document.getElementById(tileId).style = tileStyle;
      }
    }
  }
}

// Move the tiles

function tilesMove(direction) {
  // This function is called by clicking one of the 4 arrow pictures on the webpage
  if (gameStarted && !gameEnded) {
    if (direction == 'up') {
      // Move tiles upwards
      for (k = 1; k < boardSize; k++) {
        for (i = 1; i < boardSize; i++) {
          for (j = 0; j < boardSize; j++) {
            if (boardGrid[i - 1][j] == 0) {
              boardGrid[i - 1][j] = boardGrid[i][j];
              boardGrid[i][j] = 0;
            }
            else if (boardGrid[i - 1][j] == boardGrid[i][j]) {
              boardGrid[i - 1][j] = 2 * boardGrid[i][j];
              boardGrid[i][j] = 0;
            }
          }
        }
      }
    }
    else if (direction == 'down') {
      // Move tiles downwards
      for (k = 1; k < boardSize; k++) {
        for (i = boardSize - 2; i >= 0 ; i--) {
          for (j = 0; j < boardSize; j++) {
            if (boardGrid[i + 1][j] == 0) {
              boardGrid[i + 1][j] = boardGrid[i][j];
              boardGrid[i][j] = 0;
            }
            else if (boardGrid[i + 1][j] == boardGrid[i][j]) {
              boardGrid[i + 1][j] = 2 * boardGrid[i][j];
              boardGrid[i][j] = 0;
            }
          }
        }
      }
    }
    else if (direction == 'left') {
      // Move tiles leftwards
      for (k = 1; k < boardSize; k++) {
        for (j = 1; j < boardSize; j++) {
          for (i = 0; i < boardSize; i++) {
            if (boardGrid[i][j - 1] == 0) {
              boardGrid[i][j - 1] = boardGrid[i][j];
              boardGrid[i][j] = 0;
            }
            else if (boardGrid[i][j - 1] == boardGrid[i][j]) {
              boardGrid[i][j - 1] = 2 * boardGrid[i][j];
              boardGrid[i][j] = 0;
            }
          }
        }
      }
    }
    else if (direction == 'right') {
      // Move tiles rightwards
      for (k = 1; k < boardSize; k++) {
        for (j = boardSize - 2; j >= 0 ; j--) {
          for (i = 0; i < boardSize; i++) {
            if (boardGrid[i][j + 1] == 0) {
              boardGrid[i][j + 1] = boardGrid[i][j];
              boardGrid[i][j] = 0;
            }
            else if (boardGrid[i][j + 1] == boardGrid[i][j]) {
              boardGrid[i][j + 1] = 2 * boardGrid[i][j];
              boardGrid[i][j] = 0;
            }
          }
        }
      }
    }
    checkGameEnd();                                                            // check if the game is over
    if (!gameEnded) {
      tilesNew();                                                              // generate new "2" tile
    }
    boardDisplay();                                                            // display the board
    if (gameEnded) {
      // Game over
      if (gameWin == true) {
        alert("Great! You did it!");
      }
      else {
        alert("Game over! Maybe, if you try another one...");
      }
    }
  }
}

// Check if the game is over

function checkGameEnd() {
  gameWin = false; gameLose = true;
  for (i = 0; i < boardSize; i++) {
    for (j = 0; j < boardSize; j++) {
      if (boardGrid[i][j] == 2048) {
        // If there is any tile with a value of 2048, the game is over (with player wins)
        gameWin = true;
      }
      else if (boardGrid[i][j] == 0) {
        // If there remains any empty tile, the game isn't yet over (with player loses)
        gameLose = false;
      }
    }
  }
  if (gameWin || gameLose) {
    gameEnded = true;
    newX = -1; newY = -1;
  }
}
